class OriginalPicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :original_picture_license_group
  has_one :resource_picture
  has_many :pictures
  
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :artist_id, :presence => true, :numericality => true, :existence => true
  validates :md5, :presence => true, :length => {:minimum => 32, :maximum => 32}
  
  before_destroy :destroy_with_file
  
  def supply_default
  end
  
  def overwrite ar
    self.artist_id = ar.id
  end
  
  def own? ar
    if ar.is_a?(Author)
      self.artist_id == ar.artist.id
    elsif ar.is_a?(Artist)
      self.artist_id == ar.id
    else
      false
    end
  end
  
  def visible? ar
    self.own?(ar)
  end
  
  def filename
    "#{self.id}.#{self.ext}"
  end
  
  def mime_type
    "image/#{self.ext}"
  end
  
  def url
    '/original_pictures/' + filename
  end
  
  def opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.width, self.height)
    {:src => self.url, :width => tw, :height => th}
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.mylist artist_id, page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(self.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['original_pictures.artist_id = ?', artist_id], :order => 'original_pictures.updated_at desc'})
    OriginalPicture.find(:all, opt)
  end
  
  def history 
    Picture.find(:all, {:conditions => ['pictures.original_picture_id = ?', self.id], :order => 'pictures.revision desc'} )
  end
  
  def self.list_opt
    {:include => {:resource_picture => {}, :pictures => {}}}
  end
  
  def self.list_json_opt
    {:include => {:resource_picture => {}, :pictures => {}}}
  end
  
  def self.show cid, ar
    opt = {}
    opt.merge!(self.show_opt)
    res = OriginalPicture.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(ar)
    res
  end
  
  def self.edit cid, ar
    opt = {}
    opt.merge!(self.show_opt)
    res = OriginalPicture.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.own?(ar)
    res
  end
  
  def self.show_opt
    {:include => {:resource_picture => {}, :pictures => {}}}
  end
  
  def self.show_json_opt
    {:include => {:resource_picture => {}, :pictures => {}}}
  end
  
  def destroy_with_file
    PictureIO.original_picture_io.delete self.filename
    self.resource_picture.destroy
  end
  
  def store(imager)
    unless imager
      self.errors.add :base, 'illegal picture data'
      return false
    end
    res = false
    self.attributes = {:ext => imager.ext, :width => imager.width, :height => imager.height, :filesize => imager.filesize, :md5 => imager.md5}
    OriginalPicture.transaction do
      if res = self.save
        if res = PictureIO.original_picture_io.put(imager.binary, self.filename)
          res = true
        else
          self.errors.add :base, 'original picture io does not work'
          raise ActiveRecord::Rollback
        end
      end
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.original_picture_io.get self.filename, subdir
  end
  
  def self.export ar
    l = LicenseGroup.list
    op = OriginalPicture.list ar.id
    {:license_groups => l, :original_pictures => op}
  end
  
end
